@@ -29,8 +29,8 @@ impl From<io::Error> for VlqEncodingError {
 }
 
 /// Write encoded unsigned values using VLQ and signed values first with ZigZag, then using VLQ
-/// for VLQ see [[https://en.wikipedia.org/wiki/Variable-length_quantity]]
-/// for ZigZag see https://developers.google.com/protocol-buffers/docs/encoding#types
+/// for VLQ see <https://en.wikipedia.org/wiki/Variable-length_quantity> (GLE)
+/// for ZigZag see <https://developers.google.com/protocol-buffers/docs/encoding#types>
 pub trait WriteSigmaVlqExt: io::Write {
     /// Write i8 without encoding
     fn put_i8(&mut self, v: i8) -> io::Result<()> {
@@ -52,13 +52,15 @@ pub trait WriteSigmaVlqExt: io::Write {
         Self::put_u64(self, v as u64)
     }
 
-    /// Cast to u16 (with range check) and encode using VLQ
-    fn put_usize_as_u16(&mut self, v: usize) -> io::Result<()> {
+    /// Cast to u16 (panics if out of range) and encode using VLQ
+    fn put_usize_as_u16_unwrapped(&mut self, v: usize) -> io::Result<()> {
+        #[allow(clippy::unwrap_used)]
         Self::put_u16(self, u16::try_from(v).unwrap())
     }
 
-    /// Cast to u32 (with range check) and encode using VLQ
-    fn put_usize_as_u32(&mut self, v: usize) -> io::Result<()> {
+    /// Cast to u32 (panics if out of range) and encode using VLQ
+    fn put_usize_as_u32_unwrapped(&mut self, v: usize) -> io::Result<()> {
+        #[allow(clippy::unwrap_used)]
         Self::put_u32(self, u32::try_from(v).unwrap())
     }
 
@@ -72,6 +74,12 @@ pub trait WriteSigmaVlqExt: io::Write {
         Self::put_u64(self, v as u64)
     }
 
+    /// Write bytes of v directly (big-endian format)
+    fn put_u32_be_bytes(&mut self, v: u32) -> io::Result<()> {
+        self.write_all(&v.to_be_bytes())?;
+        Ok(())
+    }
+
     /// Encode using ZigZag and then VLQ.
     fn put_i64(&mut self, v: i64) -> io::Result<()> {
         Self::put_u64(self, zig_zag_encode::encode_i64(v))
@@ -84,7 +92,6 @@ pub trait WriteSigmaVlqExt: io::Write {
         let mut value = v;
         // from https://github.com/ScorexFoundation/scorex-util/blob/3dc334f68ebefbfab6d33b57f2373e80245ab34d/src/main/scala/scorex/util/serialization/VLQWriter.scala#L97-L117
         // original source: http://github.com/google/protobuf/blob/a7252bf42df8f0841cf3a0c85fdbf1a5172adecb/java/core/src/main/java/com/google/protobuf/CodedOutputStream.java#L1387
-        // see https://rosettacode.org/wiki/Variable-length_quantity for implementations in other languages
         loop {
             if (value & !0x7F) == 0 {
                 buffer[position] = value as u8;
@@ -115,14 +122,43 @@ pub trait WriteSigmaVlqExt: io::Write {
     fn put_i16_be_bytes(&mut self, v: i16) -> io::Result<()> {
         self.write_all(v.to_be_bytes().as_ref())
     }
+
+    /// Put a short string (< 256 bytes) into the writer. Writes length (as u8) and string bytes to the writer
+    fn put_short_string(&mut self, s: &str) -> io::Result<()> {
+        if s.len() > 255 {
+            return Err(io::Error::new(
+                io::ErrorKind::Unsupported,
+                "Serializing strings with more than 255 bytes is not allowed",
+            ));
+        }
+        self.put_u8(s.len() as u8)?;
+        self.write_all(s.as_bytes())?;
+        Ok(())
+    }
+
+    /// Encode an optional value
+    fn put_option<T>(
+        &mut self,
+        opt: Option<T>,
+        put_value: &dyn Fn(&mut Self, T) -> io::Result<()>,
+    ) -> io::Result<()> {
+        match opt {
+            Some(s) => {
+                self.put_u8(1)?;
+                put_value(self, s)?;
+            }
+            None => self.put_u8(0)?,
+        }
+        Ok(())
+    }
 }
 
 /// Mark all types implementing `Write` as implementing the extension.
 impl<W: io::Write + ?Sized> WriteSigmaVlqExt for W {}
 
 /// Read and decode values using VLQ (+ ZigZag for signed values) encoded and written with [`WriteSigmaVlqExt`]
-/// for VLQ see [[https://en.wikipedia.org/wiki/Variable-length_quantity]]
-/// for ZigZag see https://developers.google.com/protocol-buffers/docs/encoding#types
+/// for VLQ see <https://en.wikipedia.org/wiki/Variable-length_quantity> (GLE)
+/// for ZigZag see <https://developers.google.com/protocol-buffers/docs/encoding#types>
 pub trait ReadSigmaVlqExt: io::Read {
     /// Read i8 without decoding
     fn get_i8(&mut self) -> Result<i8, io::Error> {
@@ -172,7 +208,6 @@ pub trait ReadSigmaVlqExt: io::Read {
     fn get_u64(&mut self) -> Result<u64, VlqEncodingError> {
         // source: http://github.com/google/protobuf/blob/a7252bf42df8f0841cf3a0c85fdbf1a5172adecb/java/core/src/main/java/com/google/protobuf/CodedInputStream.java#L2653
         // for faster version see: http://github.com/google/protobuf/blob/a7252bf42df8f0841cf3a0c85fdbf1a5172adecb/java/core/src/main/java/com/google/protobuf/CodedInputStream.java#L1085
-        // see https://rosettacode.org/wiki/Variable-length_quantity for implementations in other languages
         let mut result: i64 = 0;
         let mut shift = 0;
         while shift < 64 {
@@ -196,12 +231,36 @@ pub trait ReadSigmaVlqExt: io::Read {
         bits.truncate(size);
         Ok(bits.iter().map(|x| *x).collect::<Vec<bool>>())
     }
+
+    /// Reads a string from the reader. Reads a byte (size), and the string
+    fn get_short_string(&mut self) -> Result<String, VlqEncodingError> {
+        let size_bytes = self.get_u8()?;
+        let mut bytes = vec![0u8; size_bytes as usize];
+        self.read_exact(&mut bytes)?;
+        let string = String::from_utf8(bytes).map_err(|_| VlqEncodingError::VlqDecodingFailed)?;
+        Ok(string)
+    }
+
+    /// Read and decode an optional value using supplied function
+    fn get_option<T>(
+        &mut self,
+        get_value: &dyn Fn(&mut Self) -> Result<T, VlqEncodingError>,
+    ) -> Option<T> {
+        let is_opt = self.get_u8().ok()?;
+        match is_opt {
+            1 => Some(get_value(self).ok()?),
+            // Should only ever be 0 or 1
+            _ => None,
+        }
+    }
 }
 
 /// Mark all types implementing `Read` as implementing the extension.
 impl<R: io::Read + ?Sized> ReadSigmaVlqExt for R {}
 
+#[allow(clippy::unwrap_used)]
 #[cfg(test)]
+#[allow(clippy::panic)]
 mod tests {
     // See corresponding test suite in
     // https://github.com/ScorexFoundation/scorex-util/blob/9adb6c68b8a1c00ec17730e6da11c2976a892ad8/src/test/scala/scorex/util/serialization/VLQReaderWriterSpecification.scala#L11
@@ -879,6 +938,15 @@ mod tests {
             prop_assert_eq![bits.clone(), r.get_bits(bits.len()).unwrap()];
         }
 
+        #[test]
+        fn prop_short_string_roundtrip(s in ".{1,255}".prop_filter("Filter strings that are too large", |s| s.len() < 256)) {
+            let mut w = Cursor::new(vec![]);
+            w.put_short_string(&s).unwrap();
+            let inner = w.into_inner();
+            prop_assert_eq!(inner[0] as usize, s.len());
+            prop_assert_eq!(std::str::from_utf8(&inner[1..]), Ok(&*s));
+        }
+
         #[test]
         fn arbitrary_values_list(vals in collection::vec(any::<Val>(), 0..100)) {
             let mut w = Cursor::new(vec![]);
